from PyQt5.QtWidgets import (QPushButton, QLabel, QComboBox, QLineEdit, QGridLayout, QWidget,
                             QVBoxLayout, QHBoxLayout, QRadioButton, QScrollArea, QTableWidget,
                             QTableWidgetItem, QAbstractItemView, QFrame, QFormLayout)
from PyQt5.QtGui import QFont
from PyQt5.QtCore import Qt
from fitness_tracker.user_profile.profile_db import fetch_units
from .estimator import StrengthLevelEstimator
from .exercise_standards import LiftStandards

class MainPanel(QWidget):
  def __init__(self, parent):
    super().__init__(parent)
    self.units = "kg" if fetch_units() == "metric" else "lb"
    self.CreatePanel()

  def CreatePanel(self):
    main_panel_layout = QGridLayout()
    main_panel_layout.addLayout(self.description(), 0, 0, 1, 1)
    main_panel_layout.addWidget(self.calculator(), 2, 0, 2, 1)
    main_panel_layout.addLayout(self.strength_standards(), 6, 0, 4, 1)
    self.setLayout(main_panel_layout)

  def description(self):
    description_layout = QVBoxLayout()
    description_label = QLabel("Take a quick test and see where you are standing compared to other lifters.")
    description_label.setFixedHeight(30)
    description_layout.addWidget(description_label)

    return description_layout

  def calculator(self):
    calculator_framed = QFrame()
    calculator_layout = QHBoxLayout()
    data_layout = QFormLayout()
    print_layout = QVBoxLayout()
    
    gender_age_label = QLabel("Gender/Age")
    gender_age_label.setFont(QFont("Ariel", 10))
    self.gender_combobox = QComboBox(self)
    self.gender_combobox.addItems(["Male", "Female"])
    self.age_combobox = QComboBox(self)
    self.age_combobox.addItems(["14-17", "18-23", "24-39", "40-49", "50-59", "60-69", "70-79", "80-89"]) 
    
    gender_age_layout = QHBoxLayout()
    gender_age_layout.addWidget(self.gender_combobox)
    gender_age_layout.addWidget(self.age_combobox)

    bodyweight_layout = QHBoxLayout()
    units_label = QLabel(self.units)
    bodyweight_label = QLabel("Bodyweight")
    bodyweight_label.setFont(QFont("Ariel", 10))
    self.bodyweight_line_edit = QLineEdit()
    bodyweight_layout.addWidget(self.bodyweight_line_edit)
    bodyweight_layout.addWidget(units_label)
    
    exercise_label = QLabel("Exercise")
    exercise_label.setFont(QFont("Ariel", 10))
    self.exercise_combobox = QComboBox()
    self.exercise_combobox.addItems(["Bench Press", "Deadlift", "Squat"])

    weight_layout = QHBoxLayout()
    units_label1 = QLabel(self.units)
    weight_label = QLabel("Weight")
    weight_label.setFont(QFont("Ariel", 10))
    self.weight_line_edit = QLineEdit()
    weight_layout.addWidget(self.weight_line_edit)
    weight_layout.addWidget(units_label1)
    
    self.calculate_button = QPushButton("Calculate strength")
    self.calculate_button.clicked.connect(lambda: self.calculate())
    
    data_layout.addRow(gender_age_label, gender_age_layout)
    data_layout.addRow(bodyweight_label, bodyweight_layout)
    data_layout.addRow(exercise_label, self.exercise_combobox)
    data_layout.addRow(weight_label, weight_layout)
    data_layout.addRow(self.calculate_button)

    strength_estimator_exercise_label = QLabel("Your strength level for deadlift is")
    strength_estimator_exercise_label.setFont(QFont("Ariel", 14))

    self.strength_level_exercise_label = QLabel()
    self.strength_level_exercise_label.setFont(QFont("Ariel", 14))

    self.strength_to_bodyweight_label = QLabel()
    self.strength_to_bodyweight_label.setFont(QFont("Ariel", 14))

    print_layout.addWidget(strength_estimator_exercise_label)
    print_layout.addWidget(self.strength_level_exercise_label)
    print_layout.addWidget(self.strength_to_bodyweight_label)

    framed_data_layout = QFrame()
    framed_data_layout.setFrameStyle(QFrame.StyledPanel)
    framed_data_layout.setLayout(data_layout)

    calculator_layout.addWidget(framed_data_layout)
    calculator_layout.addLayout(print_layout)

    calculator_framed.setFrameStyle(QFrame.StyledPanel)
    calculator_framed.setLayout(calculator_layout)

    return calculator_framed

  def calculate(self):
    try:
      gender = self.gender_combobox.currentText()
      age = self.age_combobox.currentText()
      bodyweight = float(self.bodyweight_line_edit.text())
      exercise = self.exercise_combobox.currentText()
      weight = float(self.weight_line_edit.text())
      estimator = StrengthLevelEstimator(gender, age, bodyweight, exercise, weight, self.units)
      strength_group = estimator.find_strength_group()
      self.strength_level_exercise_label.setText(strength_group)

      lift_weight_ratio = estimator.lift_weight_ratio()
      self.strength_to_bodyweight_label.setText(" ".join(["Your lift is", str(lift_weight_ratio), "times your bodyweight."]))
    except ValueError: # user submitted text in bodyweight or weight line edit
      pass

  def strength_standards(self):
    strength_standards_layout = QVBoxLayout()
    standards_label = QLabel("Strength Standards")
    standards_label.setFont(QFont("Ariel", 18))
    strength_standards_layout.addWidget(standards_label)

    standards_info_layout = QHBoxLayout()
    self.male_button = QRadioButton("Male")
    self.male_button.setChecked(True)
    self.male_button.toggled.connect(self.update_standards_table)
    standards_info_layout.addWidget(self.male_button)
    self.female_button = QRadioButton("Female")
    self.female_button.toggled.connect(self.update_standards_table)
    standards_info_layout.addWidget(self.female_button)

    self.table_age_combobox = QComboBox(self)
    self.table_age_combobox.addItems(["14-17", "18-23", "24-39", "40-49", "50-59", "60-69", "70-79", "80-89"])
    self.table_age_combobox.activated[str].connect(self.update_standards_table)
    self.table_age_combobox.setFixedWidth(80)
    self.table_exercise_combobox = QComboBox()
    self.table_exercise_combobox.addItems(["Bench Press", "Deadlift", "Squat"])
    self.table_exercise_combobox.setFixedWidth(120)
    self.table_exercise_combobox.activated[str].connect(self.update_standards_table)
    standards_info_layout.addWidget(self.table_age_combobox)
    standards_info_layout.addWidget(self.table_exercise_combobox)
    standards_info_layout.addStretch(1)
    standards_info_layout.setSpacing(10)

    strength_standards_layout.addLayout(standards_info_layout)
    strength_standards_layout.addWidget(self.create_standards_table())
    
    return strength_standards_layout

  def create_standards_table(self):
    standards_table_scroll_area = QScrollArea()

    self.standards_table = QTableWidget()
    self.standards_table.setRowCount(20)
    self.standards_table.setColumnCount(6)
    self.standards_table.horizontalHeader().setVisible(False)
    self.standards_table.verticalHeader().setVisible(False)
    
    exercise = self.table_exercise_combobox.currentText()
    age_range = self.table_age_combobox.currentText()
    gender = "Male" if self.male_button.isChecked() else "Female"
    fetch_standard = LiftStandards(exercise, age_range, gender, self.units).standard()
    standards_table_info = [fetch_standard[0]]
    for standard in fetch_standard[1:]:
      standards_table_info.append(list(map(str, standard)))
    
    currentRow = 0
    for row in standards_table_info:
      currentColumn = 0
      for item in row:
        table_item = QTableWidgetItem(item)
        table_item.setFlags(Qt.ItemIsEnabled)
        self.standards_table.setItem(currentRow, currentColumn, table_item)
        currentColumn += 1
      currentRow += 1

    standards_table_scroll_area.setWidget(self.standards_table)
    standards_table_scroll_area.setWidgetResizable(True)

    return standards_table_scroll_area

  def update_standards_table(self):
    exercise = self.table_exercise_combobox.currentText()
    age_range = self.table_age_combobox.currentText()
    gender = "Male" if self.male_button.isChecked() else "Female"
    fetch_standard = LiftStandards(exercise, age_range, gender, self.units).standard()
    standards_table_info = [fetch_standard[0]]
    for standard in fetch_standard[1:]:
      standards_table_info.append(list(map(str, standard)))
    
    currentRow = 0
    for row in standards_table_info:
      currentColumn = 0
      for item in row:
        table_item = QTableWidgetItem(item)
        table_item.setFlags(Qt.ItemIsEnabled)
        self.standards_table.setItem(currentRow, currentColumn, table_item)
        currentColumn += 1
      currentRow += 1



import os
from PyQt5.QtWidgets import QWidget, QLabel, QGridLayout, QPushButton, QFrame, QVBoxLayout, QPlainTextEdit, QHBoxLayout, QTextEdit
from PyQt5.QtGui import QFont, QIcon
from PyQt5.QtCore import Qt, QSize

path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "exercise_images")

class Exercise(QWidget):
  def __init__(self, parent, exercise_info):
    super().__init__(parent)
    self.exercise_info = exercise_info
    self.grid = QGridLayout()
    self.create_panel()
    self.setLayout(self.grid)

  def create_panel(self):
    self.grid.addLayout(self.create_title_and_image(), 0, 0, 1, 2)
    self.grid.addWidget(self.create_description(), 1, 0, 2, 1)
    self.grid.addWidget(self.create_muscle_targeted(), 1, 1, 1, 1)
    self.grid.addWidget(self.create_tips(), 2, 1, 1, 1)

  def create_title_and_image(self):
    layout = QVBoxLayout()
    
    label = QLabel(self.exercise_info["name"], self)
    label.setAlignment(Qt.AlignCenter)
    label.setFont(QFont("Ariel", 30))

    exercise = QPushButton(self)
    
    image_name = os.path.basename(self.exercise_info["image"][0])
    exercise.setIcon(QIcon(os.path.join(path, image_name)))
    exercise.setIconSize(QSize(200, 200))
    exercise.resize(200, 200)
    exercise.setStyleSheet("border: none")
    
    layout.addWidget(label)
    layout.addWidget(exercise)
    
    return layout

  def create_description(self):
    description_frame = QFrame()
    description_layout = QVBoxLayout()
    
    description_label = QLabel("Description", self)
    description_label.setAlignment(Qt.AlignLeft)
    
    description = QTextEdit()
    description.setHtml(self.exercise_info["description"])
    description.setReadOnly(True)

    description_layout.addWidget(description_label)
    description_layout.addWidget(description)
    description_frame.setLayout(description_layout)

    return description_frame

  def create_muscle_targeted(self):
    muscle_targeted_frame = QFrame()
    muscle_targeted_layout = QVBoxLayout()
    
    muscle_targeted_label = QLabel("Muscle Targeted", self)
    
    primary_muscles = ""
    for targeted in self.exercise_info["muscles_primary"]:
      if primary_muscles == "": primary_muscles += targeted
      else: primary_muscles += ", " + targeted
    primary_label = QLabel("Primary: %s" % primary_muscles,self)
    
    secondary_muscles = ""
    for targeted in self.exercise_info["muscles_secondary"]:
      if secondary_muscles == "": secondary_muscles += targeted
      else: secondary_muscles += ", " + targeted
    secondary_label = QLabel("Secondary: %s" % secondary_muscles, self)

    muscle_targeted_layout.addWidget(muscle_targeted_label)
    muscle_targeted_layout.addWidget(primary_label)
    muscle_targeted_layout.addWidget(secondary_label)
    muscle_targeted_frame.setLayout(muscle_targeted_layout)

    return muscle_targeted_frame

  def create_tips(self):
    tips_frame = QFrame()
    tips_layout = QVBoxLayout()

    tips_label = QLabel("Tips", self)
    tips = QPlainTextEdit()
    if len(self.exercise_info["comment"]) > 0: tips.appendPlainText(self.exercise_info["comment"])
    tips.setReadOnly(True)

    tips_layout.addWidget(tips_label)
    tips_layout.addWidget(tips)
    tips_frame.setLayout(tips_layout)

    return tips_frame



import sys
import os
import importlib
import sqlite3
from PyQt5.QtWidgets import QApplication, QMainWindow, QDesktopWidget, QVBoxLayout, QWidget, QSpacerItem, QSizeGrip, QSizePolicy
from PyQt5.QtCore import Qt, QObject, pyqtSlot
from PyQt5.QtGui import QColor, QFontDatabase
from fitness_tracker.homepage.homepage import Homepage
from fitness_tracker.user_profile.profile import Profile
from fitness_tracker.calculators.one_rep_max.one_rep_max import OneRepMaxCalculator
from fitness_tracker.calculators.body_fat.body_fat import BodyFatCalculator
from fitness_tracker.calculators.strength_estimator.strength_estimator import StrengthEstimator
from fitness_tracker.notes.big_lifts.big_lifts import BigLiftsNotes
from fitness_tracker.notes.weight_loss.weight_loss import WeightLossNotes
from fitness_tracker.notes.workouts.workouts import WorkoutsNotes
from fitness_tracker.notes.nutrition.nutrition import NutritionNotes
from fitness_tracker.statistics.strength.strength import StrengthStats
from fitness_tracker.statistics.weight_loss.weight_loss import WeightLossStats
from fitness_tracker.login.login import Login
from fitness_tracker.signup.signup import Signup
from fitness_tracker.signup.signup_questions_panel import SignupQuestions
from fitness_tracker.titlebar import TitleBar
from fitness_tracker.config import db_path

class FitnessTracker(QMainWindow):
  def __init__(self):
    super().__init__()
    self.create_window()
    self.create_db_file()
    self.cw = Homepage() if self.users_table_exists() else Login()
    self.cw.display_layout_signal.connect(lambda layout: self.display_layout(layout))
    self.layouts = {"Login": Login, "Signup": Signup, "Continue": SignupQuestions,
                    "Home": Homepage, "Profile": Profile, "Logout": Login,
                    "Big Lifts": BigLiftsNotes, "Workouts": WorkoutsNotes, "Nutrition": NutritionNotes,
                    "Weight Loss Notes": WeightLossNotes, "1 Rep Max Calculator": OneRepMaxCalculator,
                    "Body Fat Calculator": BodyFatCalculator, "Strength Estimator": StrengthEstimator,
                    "Strength Statistics": StrengthStats, "Weight Loss Stats": WeightLossStats}
    self.colorize_background()
    self.colorize_foreground()
    self.setup_borders()
    self.title_bar = TitleBar(self)
    self.setMouseTracking(True)
    main_widget = self.setup_main_layout()
    self.setCentralWidget(main_widget)

  def colorize_background(self):
    self.setAutoFillBackground(True)
    bg_palette = self.palette()
    bg_palette.setColor(self.backgroundRole(), QColor(25,23,22))
    self.setPalette(bg_palette)

  def colorize_foreground(self):
    fg_palette = self.palette()
    fg_palette.setColor(self.foregroundRole(), QColor(255,255,255))
    self.setPalette(fg_palette)

  def create_window(self):
    self.setWindowTitle("Fitness Tracker")
    self.resize(1126, 733)
    self.setMinimumSize(1126, 733)
    self.center()
    self.show()

  def setup_borders(self):
    self.setWindowFlags(Qt.FramelessWindowHint)
    self.setMouseTracking(True)
    self.show()

  def setup_main_layout(self):
    main_layout = QVBoxLayout()
    main_layout.setContentsMargins(0, 0, 0, 0)
    main_layout.setSpacing(0)
    main_layout.addWidget(self.title_bar)
    main_layout.addWidget(self.cw)
    #sizegrip = QSizeGrip(self)
    #sizegrip.setMaximumWidth(16)
    #main_layout.addWidget(sizegrip)
    main_widget = QWidget()
    main_widget.setLayout(main_layout)
    return main_widget
  
  def center(self):
    window = self.frameGeometry()
    center_point = QDesktopWidget().availableGeometry().center()
    window.moveCenter(center_point)
    self.move(window.topLeft())
  
  @pyqtSlot(str)
  def display_layout(self, layout_name):
      self.cw = self.layouts[layout_name]()
      self.cw.display_layout_signal.connect(lambda layout_name: self.display_layout(layout_name))
      self.layout = QWidget()
      self.layout = self.setup_main_layout()
      self.setCentralWidget(self.layout)

  def users_table_exists(self):
    with sqlite3.connect(db_path) as conn:
      cursor = conn.cursor()
      query = "SELECT COUNT(*) FROM sqlite_master WHERE type='table' AND name='users'"
      cursor.execute(query)
      if cursor.fetchone()[0] == 0: return False
      return True

  def create_db_file(self):
    db_file = os.path.join(os.path.dirname(importlib.util.find_spec("fitness_tracker").origin), "fitness_tracker.db")
    if not os.path.isfile(db_file): open(db_file, "w")

if __name__ == "__main__":
  app = QApplication(sys.argv)
  ft = FitnessTracker()
  sys.exit(app.exec_())



from PyQt5.QtWidgets import QWidget, QGridLayout, QHBoxLayout, QLabel, QFrame
from PyQt5.QtGui import QFont
from PyQt5.QtCore import Qt

class Header(QWidget):
  def __init__(self, parent, title):
    super().__init__(parent)
    self.title = title
    self.create_header()
  
  def create_header(self):
    grid = QGridLayout()
    self.setLayout(grid)
    
    title_layout = QHBoxLayout()
    title_label = QLabel(self.title, self)
    title_label.setAlignment(Qt.AlignCenter)
    title_label.setFont(QFont("Arial", 25))
    title_layout.addWidget(title_label)

    header_frame = QFrame()
    header_frame.setFrameStyle(QFrame.StyledPanel)
    header_layout = QHBoxLayout()

    description = QLabel("Store your workouts here. \nTrack your progress with graphs. Explore exercises in our exercises database. \nYou can create your workouts.", self)
    description.setAlignment(Qt.AlignCenter)
    header_layout.addWidget(description)
    header_frame.setLayout(header_layout)
    
    grid.addLayout(title_layout, 0, 0, 1, 1)
    grid.addWidget(header_frame, 0, 3, 1, 3)



import json
import os
from PyQt5.QtWidgets import (QWidget, QGridLayout, QFrame, QVBoxLayout, QFormLayout, QLineEdit,
                             QLabel, QPushButton, QGroupBox, QRadioButton, QHBoxLayout, QComboBox)
from PyQt5.QtGui import QFont, QCursor
from PyQt5.QtCore import Qt, pyqtSignal
from .signup_helpers import create_user_info_after_signup
from fitness_tracker.user_profile.profile_db import logged_in_user_email
from fitness_tracker.notes.nutrition.nutrition_db import create_nutrition_table, insert_calorie_goal
from fitness_tracker.notes.nutrition.calorie_goal_calculator import CalorieGoalCalculator

class SignupQuestions(QWidget):
  display_layout_signal = pyqtSignal(str)

  def __init__(self):
    super().__init__()
    self.create_panel()

  def create_panel(self):
    grid = QGridLayout()
    grid.addLayout(self.create_login(), 0, 0, 1, 1)
    self.setLayout(grid)

  def create_login(self):
    title_frame = QFrame()
    title_layout = QVBoxLayout()

    signup_label = QLabel("Signup", self)
    signup_label.setFont(QFont("Ariel", 15))
    signup_label.setFixedHeight(70)

    title_layout.addWidget(signup_label)
    title_frame.setLayout(title_layout)

    signup_frame = QFrame()
    signup_frame.setFrameStyle(QFrame.StyledPanel)

    form_layout = self.create_form_layout()
    signup_frame.setLayout(form_layout)

    wrapper_layout = QVBoxLayout()
    wrapper_layout.addWidget(title_frame)
    wrapper_layout.addWidget(signup_frame)
    return wrapper_layout

  def create_form_layout(self):
    self.form_layout = QFormLayout()

    name_label = QLabel("Name", self)
    self.name_entry = QLineEdit()
    
    age_label = QLabel("Age", self)
    self.age_line_edit = QLineEdit()

    gender = QGroupBox()
    gender_layout = QHBoxLayout()
    gender_layout.setAlignment(Qt.AlignCenter)

    gender_label = QLabel("Gender", self)
    self.male_button = QRadioButton("Male")
    self.male_button.setChecked(True)
    self.female_button = QRadioButton("Female")
    gender_layout.addWidget(self.male_button)
    gender_layout.addWidget(self.female_button)
    gender.setLayout(gender_layout)
    
    units = QGroupBox()
    units_layout = QHBoxLayout()
    units_layout.setAlignment(Qt.AlignCenter)

    units_label = QLabel("Units", self)
    self.metric_button = QRadioButton("Metric")
    self.metric_button.setChecked(True)
    self.metric_button.toggled.connect(lambda: self.change_height_input())
    self.imperial_button = QRadioButton("Imperial")
    self.imperial_button.toggled.connect(lambda: self.change_height_input())
    units_layout.addWidget(self.metric_button)
    units_layout.addWidget(self.imperial_button)
    units.setLayout(units_layout)
    
    weight_label = QLabel("Weight", self)
    self.weight_entry = QLineEdit()
    
    self.height_layout = QHBoxLayout()
    height_label = QLabel("Height", self)
    self.height_entry = QLineEdit()
    self.height_layout.addWidget(self.height_entry)

    self.form_layout.addRow(name_label, self.name_entry)
    self.form_layout.addRow(age_label, self.age_line_edit)
    self.form_layout.addRow(gender_label, gender)
    self.form_layout.addRow(units_label, units)
    self.form_layout.addRow(weight_label, self.weight_entry)
    self.form_layout.addRow(height_label, self.height_layout)

    goal_label = QLabel("Goal", self)
    goal_layout = QHBoxLayout()
    goal = QGroupBox()
    self.weight_loss_button = QRadioButton("Weight loss")
    self.weight_loss_button.setChecked(True)
    self.weight_loss_button.toggled.connect(lambda: self.hide_or_show_params_layout())

    self.maintain_weight_button = QRadioButton("Maintain weight")
    self.maintain_weight_button.toggled.connect(lambda: self.hide_or_show_params_layout())
    self.weight_gain_button = QRadioButton("Weight gain")
    self.weight_gain_button.toggled.connect(lambda: self.hide_or_show_params_layout())
    
    goal_layout.addWidget(self.weight_loss_button)
    goal_layout.addWidget(self.maintain_weight_button)
    goal_layout.addWidget(self.weight_gain_button)
    goal.setLayout(goal_layout)

    self.form_layout.addRow(goal_label, goal)
    
    self.params_label = QLabel("Goal parameters")
    self.params_layout = self.calorie_params_layout()
    self.form_layout.addRow(self.params_label, self.params_layout)

    self.signup_button = QPushButton("Signup", self)
    self.signup_button.clicked.connect(lambda: self.signup())
    self.signup_button.setCursor(QCursor(Qt.PointingHandCursor))
    
    self.form_layout.addRow(self.signup_button)

    return self.form_layout

  def change_height_input(self):
    if self.imperial_button.isChecked():
      try:
        if self.height_entry2 == None: self.height_entry2 = QLineEdit()
        self.height_layout.addWidget(self.height_entry2)
      
      except (AttributeError, RuntimeError):
        self.height_entry2 = QLineEdit()
        self.height_layout.addWidget(self.height_entry2)

    else:
      self.height_entry2.setParent(None)
      self.height_entry2.deleteLater()
  
  def hide_or_show_params_layout(self):
    if self.maintain_weight_button.isChecked():
      self.signup_button.setParent(None)
      self.delete_layout(self.params_layout)
      self.form_layout.addRow(self.signup_button)
    else:
      if not self.params_layout == None:
        self.signup_button.setParent(None)
        self.params_label.setParent(None)
        self.delete_layout(self.params_layout)
        self.params_label = QLabel("Goal parameters")
        self.params_layout = self.calorie_params_layout()
        self.form_layout.addRow(self.params_label, self.params_layout)
        self.form_layout.addRow(self.signup_button)

  def delete_layout(self, layout):
    if layout is not None:
      while layout.count():
        item = layout.takeAt(0)
        widget = item.widget()
        if widget is not None:
          widget.setParent(None)
        else:
          self.delete_layout(item.layout())

  def calorie_params_layout(self):
    params_layout = QHBoxLayout()
    
    goal_weight_layout = QVBoxLayout()
    goal_weight_label = QLabel("Goal weight")
    self.goal_weight_line_edit = QLineEdit()

    goal_weight_layout.addWidget(goal_weight_label)
    goal_weight_layout.addWidget(self.goal_weight_line_edit)
    
    activity_level_layout = QVBoxLayout()
    activity_level_label = QLabel("Activity level")
    self.activity_level = QComboBox()
    self.activity_level.addItems(["Sedentary", "Lightly active", "Moderately active", "Very active", "Extra active"])

    activity_level_layout.addWidget(activity_level_label)
    activity_level_layout.addWidget(self.activity_level)
    
    weight_per_week_layout = QVBoxLayout()
    weight_per_week_label = QLabel("Weight to lose/gain per week (kg)")
    self.weight_per_week = QComboBox()
    self.weight_per_week.addItems(["0.25", "0.5", "1"])
    
    weight_per_week_layout.addWidget(weight_per_week_label)
    weight_per_week_layout.addWidget(self.weight_per_week)

    params_layout.addLayout(goal_weight_layout)
    params_layout.addLayout(activity_level_layout)
    params_layout.addLayout(weight_per_week_layout)
    
    return params_layout

  def signup(self):
    try:
      age = str(int(self.age_line_edit.text()))
      gender = "male" if self.male_button.isChecked() else "female"
      units = "metric" if self.metric_button.isChecked() else "imperial"
      height = json.dumps([float(self.height_entry.text()),
                           float(self.height_entry2.text())]) if units == "imperial" else str(float(self.height_entry.text()))
      if self.weight_loss_button.isChecked():
        goal = "Weight loss"
      elif self.maintain_weight_button.isChecked():
        goal = "Maintain weight"
      elif self.weight_gain_button.isChecked():
        goal = "Weight gain"
      goal_params = json.dumps([self.activity_level.currentText(),
                                float(self.weight_per_week.currentText())]) if not goal == "Maintain weight" else json.dumps(["Maintain", 0])
      weight = str(float(self.weight_entry.text()))
      goal_weight = str(float(self.goal_weight_line_edit.text()))
      user_info = {"name": self.name_entry.text(),
                   "age": age,
                   "gender": gender,
                   "units": units,
                   "weight": weight,
                   "height": height,
                   "goal": goal,
                   "goalparams": goal_params,
                   "goalweight": goal_weight}
      if not gender == "" and not units == "" and not self.name_entry.text() == "" and not self.weight_entry.text() == "":
        goal_params = json.loads(goal_params)
        calorie_goal_calculator = CalorieGoalCalculator(int(age), gender, float(height), float(weight), goal_params[0], goal, goal_params[1])
        calorie_goal = calorie_goal_calculator.calculate_calorie_goal()
        email = logged_in_user_email()
        create_user_info_after_signup(user_info, email)
        create_nutrition_table()
        insert_calorie_goal(calorie_goal)
        self.display_layout_signal.emit("Home")
    except ValueError:
      pass



import os
from PyQt5.QtWidgets import QWidget, QLabel, QGridLayout, QVBoxLayout, QTableWidget, QAbstractItemView, QTableWidgetItem, QHeaderView
from PyQt5.QtGui import QFont, QPixmap
from .spoonacular import FoodDatabase

path = os.path.abspath(os.path.dirname(__file__))
food_images_path = os.path.join(path, "food_images")

class FoodPanel(QWidget):
  def __init__(self, parent, food_id):
    super().__init__()
    self.api = FoodDatabase()
    self.food_info = self.api.food_info(food_id)
    self.nutrients = {nutrient["title"]: " ".join([str(nutrient["amount"]), nutrient["unit"]])
                      for nutrient in self.food_info["nutrition"]["nutrients"]}
    self.create_panel()
    self.setStyleSheet("QLabel{color:white;}")

  def create_panel(self):
    grid = QGridLayout()
    grid.addLayout(self.create_left_section(), 0, 0)
    grid.addLayout(self.create_right_section(), 0, 1)
    self.setLayout(grid)

  def create_left_section(self):
    section_layout = QVBoxLayout()

    food_title = QLabel(self.food_info["name"].capitalize())
    food_title.setFont(QFont("Ariel", 25))
    
    food_image = QLabel()
    food_pixmap = QPixmap(os.path.join(food_images_path, self.food_info["image"]))
    food_image.setPixmap(food_pixmap)
    
    section_layout.addWidget(food_title)
    section_layout.addWidget(food_image)

    return section_layout

  def create_right_section(self):
    section_layout = QVBoxLayout()

    food_info_label = QLabel("*important food info*")
    food_info_label.setFont(QFont("Ariel", 25))

    graph = QLabel()
    graph.setStyleSheet("background-color: white;")
    graph_pixmap = QPixmap(os.path.join(path, "/chart_placeholder.png"))
    graph.setPixmap(graph_pixmap)

    nutrition_table = QTableWidget(15, 1)
    nutrition_table.setEditTriggers(QAbstractItemView.NoEditTriggers)
    nutrition_table.horizontalHeader().setVisible(False)
    nutrition_table.setStyleSheet("background-color:white;")

    amount = " ".join([str(self.food_info["amount"]), self.food_info["unit"]])
    nutrients = list(self.nutrients.keys())
    nutrient_values = list(self.nutrients.values())
    
    for i, nutrient in enumerate(nutrients):
      if nutrient == "Calories": nutrient += " (%s)" % amount
      nutrition_table.setVerticalHeaderItem(i, QTableWidgetItem(nutrient))
    
    for i, value in enumerate(nutrient_values):
      nutrition_table.setItem(i, 0, QTableWidgetItem(value))

    section_layout.addWidget(food_info_label)
    section_layout.addWidget(graph)
    section_layout.addWidget(nutrition_table)
    
    return section_layout



from .exercise_standards import LiftStandards

class StrengthLevelEstimator:
  def __init__(self, gender, age_range, bodyweight, exercise, one_RM, units):
    self.gender = gender
    self.age = age_range
    self.bodyweight = bodyweight
    self.exercise = exercise
    self.one_RM = one_RM
    self.units = units
    self.lift_standard = LiftStandards(exercise, age_range, gender, self.units).standard()

  def standard(self):
    return self.lift_standard
  
  def find_strength_group(self):
    if self.gender == "Male" and self.units == "kg": bodyweight_groups = [bodyweight_group for bodyweight_group in range(50, 145, 5)]
    elif self.gender == "Female" and self.units == "kg": bodyweight_groups = [bodyweight_group for bodyweight_group in range(40, 125, 5)]
    elif self.gender == "Male" and self.units == "lb": bodyweight_groups = [bodyweight_group for bodyweight_group in range(110, 320, 10)]
    elif self.gender == "Female" and self.units == "lb": bodyweight_groups = [bodyweight_group for bodyweight_group in range(90, 270, 10)]

    bodyweight_group = self.binary_search(bodyweight_groups, self.bodyweight)
    for standard in self.lift_standard:
      if standard[0] == bodyweight_group: bodyweight_standard = standard
    exercise_weight = bodyweight_standard.index(self.binary_search(bodyweight_standard, self.one_RM))
    if exercise_weight == 0: return self.lift_standard[0][1]
    else: return self.lift_standard[0][exercise_weight]

  def binary_search(self, arr, target):
    low = 0
    high = len(arr)-1

    if target <= arr[0]: return arr[0]
    if target >= arr[-1]: return arr[-1]

    while low < high:
      mid = (low+high)//2
      if arr[mid] == target: return arr[mid]
      elif target < arr[mid]:
        if target > arr[mid-1]:
          if target-arr[mid-1] >= arr[mid]-target: return arr[mid]
          else: return arr[mid-1]
        high = mid
      else:
        if mid < len(arr)-1 and target < arr[mid+1]:
          if target-arr[mid] >= arr[mid+1]-target: return arr[mid+1]
          else: return arr[mid]
        low = mid+1
    return arr[mid]

  def lift_weight_ratio(self):
    return round(self.one_RM/self.bodyweight, 2)



from PyQt5.QtWidgets import QWidget, QGridLayout, QFrame, QVBoxLayout, QFormLayout, QLineEdit, QLabel, QPushButton
from PyQt5.QtGui import QFont, QCursor
from PyQt5.QtCore import Qt, pyqtSignal
from .login_helpers import check_password, fetch_user_info

class MainPanel(QWidget):
  emit_layout_name = pyqtSignal(str)

  def __init__(self):
    super().__init__()
    self.create_panel()

  def create_panel(self):
    grid = QGridLayout()
    grid.addLayout(self.create_login(), 0, 0, 1, 1)
    self.setLayout(grid)

  def create_login(self):
    title_frame = QFrame()
    title_layout = QVBoxLayout()

    login_label = QLabel("Login", self)
    login_label.setFont(QFont("Ariel", 15))
    login_label.setFixedHeight(70)

    title_layout.addWidget(login_label)
    title_frame.setLayout(title_layout)

    login_frame = QFrame()
    login_frame.setFrameStyle(QFrame.StyledPanel)

    form_layout = self.create_form_layout()
    login_frame.setLayout(form_layout)

    wrapper_layout = QVBoxLayout()
    wrapper_layout.addWidget(title_frame)
    wrapper_layout.addWidget(login_frame)
    return wrapper_layout

  def create_form_layout(self):
    form_layout = QFormLayout()

    email_label = QLabel("Email", self)
    self.email_entry = QLineEdit()
    
    password_label = QLabel("Password", self)
    self.password_entry = QLineEdit()
    self.password_entry.setEchoMode(QLineEdit.Password)
    
    self.login_button = QPushButton("Login", self)
    self.login_button.clicked.connect(lambda: self.login())
    self.login_button.setCursor(QCursor(Qt.PointingHandCursor))
    
    signup_label = QLabel("Don't have an account?")
    self.signup_button = QPushButton("Signup", self)
    self.signup_button.setCursor(QCursor(Qt.PointingHandCursor))
    self.signup_button.clicked.connect(lambda: self.emit_layout_name.emit(self.signup_button.text()))

    form_layout.addRow(email_label, self.email_entry)
    form_layout.addRow(password_label, self.password_entry)
    form_layout.addRow(self.login_button)
    form_layout.addRow(signup_label, self.signup_button)
    return form_layout

  def login(self):
    email = self.email_entry.text()
    password = self.password_entry.text()
    if check_password(email, password):
      fetch_user_info(email, password)
      self.emit_layout_name.emit("Home")


from PyQt5.QtWidgets import QWidget, QLabel, QGridLayout
from PyQt5.QtGui import QFont
from PyQt5.QtCore import pyqtSignal, pyqtSlot
from .notes_panel import NotesPanel
from .food_database_panel import FoodDatabasePanel
from .search_results_panel import SearchResultsPanel
from .food_panel import FoodPanel
from fitness_tracker.homepage.side_panel import SidePanel
from .header import Header

class NutritionNotes(QWidget):
  display_layout_signal = pyqtSignal(str)

  def __init__(self):
    super().__init__()
    self.notes_panel = NotesPanel(self)
    self.header = Header(self)
    self.side_panel = SidePanel(self)
    self.side_panel.emit_layout_name.connect(lambda layout_name: self.emit_display_layout_signal(layout_name))
    self.create_grid()
    self.header.change_layout_signal.connect(lambda name: self.change_layout(name))
       
  def create_grid(self):
    self.grid = QGridLayout()
    self.grid.setContentsMargins(0, 0, 0, 0)
    self.grid.addWidget(self.side_panel, 0, 0, 1, 1)
    #self.grid.addWidget(self.header, 0, 1, 1, 2)
    self.grid.addWidget(self.notes_panel, 0, 1, 1, 1)
    self.setLayout(self.grid)
  
  @pyqtSlot(str)
  def emit_display_layout_signal(self, layout_name):
    self.display_layout_signal.emit(layout_name)
  
  @pyqtSlot(str)
  def change_layout(self, name):
    grid_items = list(reversed([type(self.grid.itemAt(i).widget()).__name__ for i in reversed(range(self.grid.count()))]))
    if name == "Food Database" and not "FoodDatabasePanel" in grid_items or name == "Return to FoodDatabase":
      if "NotesPanel" in grid_items or "SearchResultsPanel" in grid_items or "FoodPanel" in grid_items:
        self.grid.itemAt(2).widget().setParent(None)
        self.food_database_panel = FoodDatabasePanel(self)
        self.food_database_panel.emit_search_results.connect(lambda search_results: self.save_search_results(search_results))
        self.food_database_panel.search_signal.connect(lambda name: self.change_layout(name))
        self.grid.addWidget(self.food_database_panel, 1, 1, 8, 3)
        self.header.set_current_layout_button(name)

    elif name == "Notes" and not "NotesPanel" in grid_items:
      if "FoodDatabasePanel" in grid_items or "SearchResultsPanel" in grid_items or "FoodPanel" in grid_items:
       self.grid.itemAt(2).widget().setParent(None)
       self.grid.addWidget(self.notes_panel, 1, 1, 8, 3)
       self.header.set_current_layout_button(name)

    elif name == "Search" and "FoodDatabasePanel" in grid_items:
      self.grid.itemAt(2).widget().setParent(None)  
      self.search_results_panel = SearchResultsPanel(self, self.search_results)
      self.search_results_panel.return_to_food_db_signal.connect(lambda name: self.change_layout(name))
      self.search_results_panel.food_panel_signal.connect(lambda food_id: self.show_food_panel(food_id))
      self.grid.addWidget(self.search_results_panel, 1, 1, 8, 3)
  
  @pyqtSlot(object)
  def save_search_results(self, search_results):
    self.search_results = search_results

  @pyqtSlot(int)
  def show_food_panel(self, food_id):
    self.grid.itemAt(2).widget().setParent(None)
    self.food_panel = FoodPanel(self, food_id)
    self.grid.addWidget(self.food_panel, 1, 1, 8, 3)



import json
from PyQt5.QtWidgets import QPushButton, QGridLayout, QVBoxLayout, QWidget, QLabel, QComboBox, QFrame, QHBoxLayout
from PyQt5.QtGui import QFont, QColor
from PyQt5.QtCore import Qt, pyqtSlot
from .preferred_lifts import PreferredLifts
from .update_1RM_window import Update1RMWindow
from .update_lifts_for_reps_window import UpdateLiftsForRepsWindow
from .lift_history import LiftHistory
from .big_lifts_db import (create_big_lifts_table, insert_default_values, table_is_empty,
                           fetch_units_from_big_lifts, fetch_one_rep_maxes, fetch_lifts_for_reps,
                           update_big_lifts_units, update_lifts_for_reps, update_1RM_lifts,
                           convert_lift_history_weight, fetch_preferred_lifts, clear_one_rep_maxes, clear_lifts_for_reps)
from fitness_tracker.user_profile.profile_db import fetch_units
from fitness_tracker.common.units_conversion import kg_to_pounds, pounds_to_kg

class MainPanel(QWidget):
  def __init__(self, parent):
    super().__init__(parent)
    self.setStyleSheet("""
    QWidget{
      color:#c7c7c7;
      font-weight: bold;
    }
    QPushButton{
      background-color: rgba(0, 0, 0, 0);
      border: 1px solid;
      font-size: 18px;
      font-weight: bold;
      border-color: #808080;
      min-height: 28px;
      white-space:nowrap;
      text-align: left;
      padding-left: 5%;
      font-family: Ubuntu;
    }
    QPushButton:hover:!pressed{
      border: 2px solid;
      border-color: #747474;
    }
    QPushButton:pressed{
      border: 2px solid;
      background-color: #323232;
      border-color: #6C6C6C;
    }
    QComboBox{
      border-radius: 4px;
      font-size: 18px;
      font-weight: bold;
      white-space:nowrap;
      text-align: left;
      padding-left: 5%;
      font-family: Ubuntu;
      min-height: 28px;
      background-color: #440D0F;
    }
    QComboBox:down-arrow{
      width: 24.54px;
      height: 10px;
      background: #d3d3d3; 
      opacity:0
    }
    QComboBox:drop-down{
      background-color: #440D0F;
      border: 0px;
      opacity:0;
      border-radius: 0px;
    }
    QComboBox:hover:!pressed{
      background-color: #5D1A1D;
    }
    QComboBox:pressed{
      background-color: #551812;
    }
    """)
    create_big_lifts_table()
    if table_is_empty(): insert_default_values()
    
    self.units = "kg" if fetch_units() == "metric" else "lb"
    big_lifts_units = "kg" if fetch_units_from_big_lifts() == "metric" else "lb"
    
    one_rep_maxes = json.loads(fetch_one_rep_maxes())
    lifts_for_reps = json.loads(fetch_lifts_for_reps())

    self.one_RM = [[lift, " ".join([weight, self.units])] for lift, weight in one_rep_maxes.items()]
    self.lifts_reps = [[lift, " ".join(["x".join(weight), self.units])] for lift, weight in lifts_for_reps.items()]
    
    if not self.units == big_lifts_units:
      update_big_lifts_units()
      if self.units == "kg":
        self.one_RM = [[lift[0], " ".join([str(pounds_to_kg(float(lift[1].split(" ")[0]))), self.units])] for lift in self.one_RM]
        self.lifts_reps = [[lift[0], " ".join(["x".join([lift[1].split("x")[0],
                                              str(pounds_to_kg(float(lift[1].split("x")[1].split(" ")[0])))]),
                                              self.units])] for lift in self.lifts_reps]
      elif self.units == "lb":
        self.one_RM = [[lift[0], " ".join([str(kg_to_pounds(float(lift[1].split(" ")[0]))), self.units])] for lift in self.one_RM]
        self.lifts_reps = [[lift[0], " ".join(["x".join([lift[1].split("x")[0],
                                              str(kg_to_pounds(float(lift[1].split("x")[1].split(" ")[0])))]),
                                              self.units])] for lift in self.lifts_reps]
      
      new_one_RM_lifts = {lift[0]:lift[1].split(" ")[0] for lift in self.one_RM}
      new_lifts_for_reps = {lift[0]: [lift[1].split("x")[0], lift[1].split("x")[1].split(" ")[0]] for lift in self.lifts_reps}
      
      update_1RM_lifts(new_one_RM_lifts)
      update_lifts_for_reps(new_lifts_for_reps)
      convert_lift_history_weight(self.units)

    self.lift_history_window = LiftHistory()
    self.lift_history_window.setGeometry(100, 200, 300, 300)   
    
    self.plists_window = PreferredLifts()
    self.plists_window.change_lifts_signal.connect(self.changed_preferred_lifts)
    
    self.update_1RM_window = Update1RMWindow()
    self.update_1RM_window.change_1RM_lifts_signal.connect(self.changed_1RM_lifts)
    self.update_1RM_window.history_signal.connect(
    lambda signal: self.lift_history_window.create_history(signal))
    
    self.lifts_for_reps = UpdateLiftsForRepsWindow()
    self.lifts_for_reps.change_lifts_for_reps_signal.connect(self.changed_lifts_for_reps)
    self.lifts_for_reps.history_signal.connect(lambda signal: self.lift_history_window.create_history(signal))

    self.create_panel()

  def create_panel(self):
    main_panel_grid = QGridLayout()
    main_panel_grid.addLayout(self.description(), 0, 0, 1, 1)
    main_panel_grid.addWidget(self.create_time_graph(), 1, 0, 4, 1)
    main_panel_grid.addLayout(self.create_bottom_layout(), 5, 0, 3, 1)
    main_panel_grid.addLayout(self.create_function_buttons(), 8, 0, 1, 1)
    self.setLayout(main_panel_grid)

  def description(self):
    panel_description = QVBoxLayout()
    desc_font = QFont("Ubuntu", 12)
    description_label = QLabel("Keep notes and track progress of your preferred big lifts here.", self)
    description_label.setFont(desc_font)
    description_label.setFixedHeight(20)
    panel_description.addWidget(description_label)
    return panel_description

  def create_time_graph(self):
    time_graph = QVBoxLayout()
    timeline_combobox = QComboBox(self)
    timeline_combobox.addItems(["Daily", "Weekly", "Monthly", "Yearly"])

    time_graph_graph = QWidget(self)
    time_graph_graph.setStyleSheet("background-color: red")

    time_graph.addWidget(time_graph_graph)
    time_graph.addWidget(timeline_combobox)
    framed_graph = QFrame(self)
    framed_graph.setObjectName("graphObj")
    framed_graph.setFrameStyle(QFrame.Box)
    framed_graph.setLineWidth(3)
    framed_graph.setStyleSheet("""#graphObj {color: #322d2d;}""")
    framed_graph.setLayout(time_graph)

    return framed_graph

  def create_bottom_layout(self):
    bottom_layout = QHBoxLayout()
    bottom_layout.addWidget(self.create_one_rep_max())
    bottom_layout.addWidget(self.create_lifts_for_reps())
    return bottom_layout

  def create_one_rep_max(self):
    orm_panel = QVBoxLayout()
    main_label = QLabel("One Rep Max")
    main_label.setFont(QFont("Ariel", 18, weight=QFont.Bold))

    self.horizontal_press_label_ORM = QLabel(": ".join(self.one_RM[0]))
    self.horizontal_press_label_ORM.setFont(QFont("Ariel", 10))

    self.floor_pull_label_ORM = QLabel(": ".join(self.one_RM[1]))
    self.floor_pull_label_ORM.setFont(QFont("Ariel", 10))

    self.squat_label_ORM = QLabel(": ".join(self.one_RM[2]))
    self.squat_label_ORM.setFont(QFont("Ariel", 10))

    self.vertical_press_label_ORM = QLabel(": ".join(self.one_RM[3]))
    self.vertical_press_label_ORM.setFont(QFont("Ariel", 10))

    orm_buttons = QHBoxLayout()
    update_button = QPushButton("Update")
    update_button.clicked.connect(lambda: self.update_1RM_window.show())
    clear_button = QPushButton("Clear")
    clear_button.clicked.connect(lambda: self.clear_one_rep_maxes())
    orm_buttons.addWidget(update_button)
    orm_buttons.addWidget(clear_button)

    orm_panel.addWidget(main_label)
    orm_panel.addWidget(self.horizontal_press_label_ORM)
    orm_panel.addWidget(self.floor_pull_label_ORM)
    orm_panel.addWidget(self.squat_label_ORM)
    orm_panel.addWidget(self.vertical_press_label_ORM)
    orm_panel.addLayout(orm_buttons)

    orm_panel.setSpacing(5)
    framed_layout = QFrame()
    framed_layout.setObjectName("graphObj")
    framed_layout.setFrameStyle(QFrame.Box)
    framed_layout.setLineWidth(3)
    framed_layout.setStyleSheet("""#graphObj {color: #322d2d;}""")
    framed_layout.setLayout(orm_panel)

    return framed_layout

  def create_lifts_for_reps(self):
    reps_panel = QVBoxLayout()
    main_label = QLabel("Lifts For Reps")
    main_label.setFont(QFont("Ariel", 18, weight=QFont.Bold))

    self.horizontal_press_label_reps = QLabel(": ".join(self.lifts_reps[0]))
    self.horizontal_press_label_reps.setFont(QFont("Ariel", 10))

    self.floor_pull_label_reps = QLabel(": ".join(self.lifts_reps[1]))
    self.floor_pull_label_reps.setFont(QFont("Ariel", 10))

    self.squat_label_reps = QLabel(": ".join(self.lifts_reps[2]))
    self.squat_label_reps.setFont(QFont("Ariel", 10))

    self.vertical_press_label_reps = QLabel(": ".join(self.lifts_reps[3]))
    self.vertical_press_label_reps.setFont(QFont("Ariel", 10))

    reps_buttons = QHBoxLayout()
    update_button = QPushButton("Update")
    update_button.clicked.connect(lambda: self.lifts_for_reps.show())
    clear_button = QPushButton("Clear")
    clear_button.clicked.connect(lambda: self.clear_lifts_for_reps())
    reps_buttons.addWidget(update_button)
    reps_buttons.addWidget(clear_button)

    reps_panel.addWidget(main_label)
    reps_panel.addWidget(self.horizontal_press_label_reps)
    reps_panel.addWidget(self.floor_pull_label_reps)
    reps_panel.addWidget(self.squat_label_reps)
    reps_panel.addWidget(self.vertical_press_label_reps)
    reps_panel.addLayout(reps_buttons)

    framed_layout = QFrame()
    framed_layout.setObjectName("graphObj")
    framed_layout.setFrameStyle(QFrame.Box)
    framed_layout.setLineWidth(3)
    framed_layout.setStyleSheet("""#graphObj {color: #322d2d;}""")
    framed_layout.setLayout(reps_panel)

    return framed_layout

  def create_function_buttons(self):
    buttons_panel = QHBoxLayout()
    lift_history_button = QPushButton()
    lift_history_button.setText("Lift History")
    lift_history_button.clicked.connect(lambda: self.lift_history_window.show())
    preferred_lists_button = QPushButton()
    preferred_lists_button.setText("Preferred Lifts")
    preferred_lists_button.clicked.connect(lambda: self.plists_window.show())

    buttons_panel.addWidget(lift_history_button)
    buttons_panel.addWidget(preferred_lists_button)
    return buttons_panel
  
  @pyqtSlot(bool)
  def changed_preferred_lifts(self, changed):
    if changed:
      fetch_lifts = fetch_preferred_lifts()
      parsed_lifts = list(json.loads(fetch_lifts).values())
      one_RM_labels = [self.horizontal_press_label_ORM, self.floor_pull_label_ORM,
                       self.squat_label_ORM, self.vertical_press_label_ORM]

      lifts_for_reps_labels = [self.horizontal_press_label_reps, self.floor_pull_label_reps,
                               self.squat_label_reps, self.vertical_press_label_reps]
      
      for i, label in enumerate(one_RM_labels):
        label_text = label.text().split(":")
        label_text[0] = parsed_lifts[i]
        label.setText(": ".join(label_text))

      for i, label in enumerate(lifts_for_reps_labels):
        label_text = label.text().split(":")
        label_text[0] = parsed_lifts[i]
        label.setText(": ".join(label_text))
  
  @pyqtSlot(bool)
  def changed_1RM_lifts(self, changed):
    if changed:
      fetch_weight = list(json.loads(fetch_one_rep_maxes()).values())
      self.set_1RM_labels_text(fetch_weight)

  @pyqtSlot(bool)
  def changed_lifts_for_reps(self, changed):
    if changed:
      fetch_reps_and_weight = list(json.loads(fetch_lifts_for_reps()).values())
      self.set_lifts_for_reps_labels_text(fetch_reps_and_weight)
  
  def set_lifts_for_reps_labels_text(self, text):
    lifts_for_reps_labels = [self.horizontal_press_label_reps, self.floor_pull_label_reps,
                             self.squat_label_reps, self.vertical_press_label_reps]
      
    for i, label in enumerate(lifts_for_reps_labels):
      label_text = label.text().split(": ")
      label_text[1] = " ".join(["x".join(text[i]), self.units])
      label.setText(": ".join(label_text))
  
  def set_1RM_labels_text(self, text):
    one_RM_labels = [self.horizontal_press_label_ORM, self.floor_pull_label_ORM,
                       self.squat_label_ORM, self.vertical_press_label_ORM]
    for i, label in enumerate(one_RM_labels):
      label_text = label.text().split(": ")
      label_text[1] = " ".join([text[i], self.units])
      label.setText(": ".join(label_text))

  def clear_one_rep_maxes(self):
    clear_one_rep_maxes()
    fetch_weight = list(json.loads(fetch_one_rep_maxes()).values())
    self.set_1RM_labels_text(fetch_weight)
    self.update_1RM_window.set_line_edit_values()
   
  def clear_lifts_for_reps(self):
    clear_lifts_for_reps()
    fetch_reps_and_weight = list(json.loads(fetch_lifts_for_reps()).values())
    self.set_lifts_for_reps_labels_text(fetch_reps_and_weight)
    self.lifts_for_reps.set_line_edit_values()



import math

# USC - inches
# SI - centimeters

# BMI Helpers
def BMI_USC(weight, height):
  return 703*(weight/height**2)

def BMI_SI(weight, height):
  # height/100, convert passed height to meters
  return weight/(height/100)**2

# U.S Navy Method - Male
def male_navy_USC(waist, neck, height):
  return 86.010*math.log10(waist-neck)-70.041*math.log10(height)+36.76

def male_navy_SI(waist, neck, height):
  return 495/((1.0324-0.19077*math.log10(waist-neck))+0.15456*math.log10(height))-450

# U.S Navy Method - Female
def female_navy_USC(waist, neck, height, hip):
  return 163.205*math.log10(waist+hip-neck)-97.684*math.log10(height)-78.387

def female_navy_SI(waist, neck, height, hip):
  return 495/(1.29579-0.35004*math.log10(waist+hip-neck)+0.22100*math.log10(height))-450

# BMI Method - Male
def male_BMI_USC(weight, height, age):
  return 1.20*BMI_USC(weight, height)+0.23*age-16.2

def male_BMI_SI(weight, height, age):
  return 1.20*BMI_SI(weight, height)+0.23*age-16.2

# BMI Method - Female
def female_BMI_USC(weight, height, age):
  return 1.20*BMI_USC(weight, height)+0.23*age-5.4

def female_BMI_SI(weight, height, age):
  return 1.20*BMI_SI(weight, height)+0.23*age-5.4

def fat_mass(body_fat, weight):
  return body_fat/100*weight

def lean_body_mass(weight, fat_mass):
  return weight-fat_mass



import os
from PyQt5.QtWidgets import QWidget, QHBoxLayout, QLabel, QPushButton, QSpacerItem, QSizePolicy, QSizeGrip
from PyQt5.QtGui import QFont, QIcon, QPainter, QBrush, QColor
from PyQt5.QtCore import QPoint, Qt, QSize
from fitness_tracker.user_profile.profile_db import fetch_username

path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "icons")

icons = {"close": os.path.join(path, "close.bmp"),
         "min": os.path.join(path, "min.bmp"),
         "max": os.path.join(path, "max.bmp"),
         "small": os.path.join(path, "small.bmp"),
         "settings": os.path.join(path, "settings.png")}

class TitleBar(QWidget):
    def __init__(self, parent):
        super(TitleBar, self).__init__()
        self.parent = parent
        self.layout = QHBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.title = QLabel("Fitness Tracker")
        self.username = fetch_username()
        if self.username == None:
          self.version = QLabel()
        else:
          self.version = QLabel("-" + self.username)
        self.setFixedHeight(25)

        self.gridlayout = QHBoxLayout()
        self.gridlayout.setContentsMargins(0, 0, 0, 0)

        slight_indent = QSpacerItem(4, 0, QSizePolicy.Minimum, QSizePolicy.Expanding)
        version_space = QSpacerItem(20, 0, QSizePolicy.Minimum, QSizePolicy.Expanding)

        title_font_size = 14
        title_font = QFont('Arial', title_font_size)
        self.title.setFont(title_font)

        self.version.setFont(title_font)
        self.version.setStyleSheet("color: #A5A5A5;")

        icon_max_size_x = 32
        icon_max_size_y = 24

        self.close_icon = QIcon(icons["close"])
        self.close_button = QPushButton()
        self.close_button.clicked.connect(self.close_click)
        self.close_button.setIcon(self.close_icon)
        self.close_button.setMaximumSize(icon_max_size_x, icon_max_size_y)
        self.close_button.setStyleSheet(
            "QPushButton{ background-color: rgba(237, 17, 17, 0); }"
            "QPushButton:hover{ background-color: rgba(237, 17, 17, 75); }"
            "QPushButton:pressed{ background-color: rgba(255, 0, 0, 100); }")

        self.minimise_icon = QIcon(icons["min"])
        self.minimise_button = QPushButton()
        self.minimise_button.clicked.connect(self.min_click)
        self.minimise_button.setIcon(self.minimise_icon)
        self.minimise_button.setMaximumSize(icon_max_size_x, icon_max_size_y)
        self.minimise_button.setStyleSheet(
            "QPushButton{ background-color: rgba(255, 255, 255, 0); }"
            "QPushButton:hover{ background-color: rgba(255, 255, 255, 70); }"
            "QPushButton:pressed{ background-color: rgba(255, 255, 255, 40); }")

        self.maximize_icon = QIcon(icons["max"])
        self.unmax_icon = QIcon(icons["small"])
        self.maximize_button = QPushButton()
        self.maximize_button.setIcon(self.maximize_icon)
        self.maximize_button.clicked.connect(self.max_click)
        self.maximize_button.setMaximumSize(icon_max_size_x, icon_max_size_y)
        self.maximize_button.setStyleSheet(
            "QPushButton{ background-color: rgba(255, 255, 255, 0); }"
            "QPushButton:hover{ background-color: rgba(255, 255, 255, 70); }"
            "QPushButton:pressed{ background-color: rgba(255, 255, 255, 40); }")

        self.settings_icon = QIcon(icons["settings"])
        self.settings_button = QPushButton()
        self.settings_button.setIcon(self.settings_icon)
        self.settings_button.setMaximumSize(icon_max_size_x, icon_max_size_y)
        self.settings_button.setStyleSheet(
            "QPushButton{ background-color: rgba(255, 255, 255, 0); }"
            "QPushButton:hover{ background-color: rgba(255, 255, 255, 70); }"
            "QPushButton:pressed{ background-color: rgba(255, 255, 255, 40); }")

        self.layout.addItem(slight_indent)
        self.layout.addWidget(self.title)
        self.layout.addItem(version_space)
        self.layout.addWidget(self.version)
        self.layout.addStretch(-1)

        self.gridlayout.addLayout(self.layout)
        self.gridlayout.addStretch(1)
        self.gridlayout.setSpacing(0)
        self.gridlayout.addWidget(self.settings_button)
        self.gridlayout.addWidget(self.minimise_button)
        self.gridlayout.addWidget(self.maximize_button)
        self.gridlayout.addWidget(self.close_button)
        self.setLayout(self.gridlayout)

        self.start = QPoint(0, 0)
        self.pressing = False
        self.maximized = False

    def mousePressEvent(self, event):
        self.start = self.mapToGlobal(event.pos())
        self.pressing = True

    def mouseMoveEvent(self, event):
        if self.pressing & self.maximized is False:
            self.end = self.mapToGlobal(event.pos())
            self.movement = self.end - self.start
            self.parent.setGeometry(self.mapToGlobal(self.movement).x(),
            self.mapToGlobal(self.movement).y(),
            self.parent.width(),
            self.parent.height())
            self.start = self.end

    def mouseReleaseEvent(self, QMouseEvent):
        self.pressing = False

    def close_click(self):
        self.parent.close()

    def max_click(self):
        if self.maximized:
            self.maximized = False
            self.parent.showNormal()
            self.maximize_button.setIcon(self.maximize_icon)
        else:
            self.maximized = True
            self.maximize_button.setIcon(self.unmax_icon)
            self.parent.showMaximized()

    def min_click(self):
        self.parent.showMinimized()

    def paintEvent(self, event):
        painter = QPainter()
        painter.begin(self)
        painter.setRenderHint(QPainter.Antialiasing)
        painter.fillRect(event.rect(), QBrush(QColor(68, 13, 15, 255)))



from PyQt5.QtWidgets import QWidget, QLabel, QGridLayout, QPushButton, QVBoxLayout
from PyQt5.QtCore import Qt, QFileInfo, QSize
from PyQt5.QtGui import QFont, QIcon, QCursor

path = QFileInfo(__file__).absolutePath()
icon_size = QSize(24, 24)

class SidePanel(QWidget):
  def __init__(self, parent, controller):
    super().__init__(parent)
    self.controller = controller
    self.create_panel()
    self.setStyleSheet("QPushButton{text-align: left}") 
  
  def create_panel(self):
    general_layout = QVBoxLayout()
    home_button = QPushButton(QIcon("".join([path, "/icons/home.png"])), "Home", self)
    home_button.clicked.connect(lambda: self.controller.display_layout(home_button.text()))
    user_button = QPushButton(QIcon("".join([path, "/icons/user.png"])), "User Data and Settings", self)
    user_button.clicked.connect(lambda: self.controller.display_layout(user_button.text()))
    
    notes_layout = QVBoxLayout()
    notes_label = QLabel("Notes", self)
    notes_label.setFont(QFont("Ariel", 15))
    notes_label.setFixedSize(100, 20)
    lifts_button = QPushButton(QIcon("".join([path, "/icons/big_lifts.png"])), "Big Lifts", self)
    lifts_button.clicked.connect(lambda: self.controller.display_layout(lifts_button.text()))
    workouts_button = QPushButton(QIcon("".join([path, "/icons/workouts.png"])), "Workouts", self)
    workouts_button.clicked.connect(lambda: self.controller.display_layout(workouts_button.text()))
    calories_button = QPushButton(QIcon("".join([path, "/icons/calories.png"])) ,"Nutrition", self)
    calories_button.clicked.connect(lambda: self.controller.display_layout(calories_button.text()))
    weight_notes_button = QPushButton(QIcon("".join([path, "/icons/weight_loss.png"])), "Weight Loss Notes", self)
    weight_notes_button.clicked.connect(lambda: self.controller.display_layout(weight_notes_button.text()))
    
    calculators_layout = QVBoxLayout()
    calculators_label = QLabel("Calculators", self)
    calculators_label.setFont(QFont("Ariel", 15))
    calculators_label.setFixedSize(120, 20)
    max_button = QPushButton(QIcon("".join([path, "/icons/one_rep_max.png"])), "1 Rep Max", self)
    max_button.clicked.connect(lambda: self.controller.display_layout(max_button.text()))
    fat_button = QPushButton(QIcon("".join([path, "/icons/body_fat.png"])), "Body Fat", self)
    fat_button.clicked.connect(lambda: self.controller.display_layout(fat_button.text()))
    strength_es_button = QPushButton(QIcon("".join([path, "/icons/strength_es.png"])), "Strength Estimator", self)
    strength_es_button.clicked.connect(lambda: self.controller.display_layout(strength_es_button.text()))
    
    stats_layout = QVBoxLayout()
    stats_label = QLabel("Statistics", self)
    stats_label.setFont(QFont("Ariel", 15))
    stats_label.setFixedSize(100, 20)
    strength_button = QPushButton(QIcon("".join([path, "/icons/strength.png"])), "Strength", self)
    strength_button.clicked.connect(lambda: self.controller.display_layout(strength_button.text()))
    weight_loss_button = QPushButton(QIcon("".join([path, "/icons/weight_loss.png"])), "Weight Loss", self)
    weight_loss_button.clicked.connect(lambda: self.controller.display_layout(weight_loss_button.text()))
    
    layouts = {general_layout: [home_button, user_button], 
               notes_layout: [notes_label, lifts_button, workouts_button, calories_button, weight_notes_button], 
               calculators_layout: [calculators_label, max_button, fat_button, strength_es_button],
               stats_layout: [stats_label, strength_button, weight_loss_button]}
    
    grid = QGridLayout()
    self.setLayout(grid)
    
    for i, (key, value) in enumerate(layouts.items()):
      for widget in value:
        key.addWidget(widget)
        if isinstance(widget, QPushButton):
          widget.setIconSize(icon_size)
          widget.setCursor(QCursor(Qt.PointingHandCursor))
      grid.addLayout(key, i, 0)



from PyQt5.QtWidgets import QWidget, QLabel
from PyQt5.QtGui import QFont
from PyQt5.QtCore import pyqtSignal, pyqtSlot
from fitness_tracker.homepage.side_panel import SidePanel

class StrengthStats(QWidget):
  display_layout_signal = pyqtSignal(str)

  def __init__(self):
    super().__init__()
    label = QLabel("Strength Statistics", self)
    label.move(300, 300)
    label.setFont(QFont("Ariel", 30))
    self.side_panel = SidePanel(self)
    self.side_panel.emit_layout_name.connect(lambda layout_name: self.emit_display_layout_signal(layout_name))

  @pyqtSlot(str)
  def emit_display_layout_signal(self, layout_name):
    self.display_layout_signal.emit(layout_name)



import os
from setuptools import setup, find_packages

directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(directory, 'README.md'), encoding='utf-8') as f:
  long_description = f.read()

setup(
  name="fitness_tracker",
  version="0.2.0",
  author="Jurica Runtas, Kristijan Milić",
  url="https://github.com/JuricaRT/fitness_tracker",
  license="MIT",
  description="Fitness Tracker is a tool that offers a better way of tracking your fitness progress.",
  long_description=long_description,
  long_description_content_type="text/markdown",
  packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
  include_package_data=True,
  classifiers=["Programming Language :: Python :: 3",
               "License :: OSI Approved :: MIT License",
               "Operating System :: OS Independent"],
  install_requires=["PyQt5", "requests", "psycopg2",
                    "matplotlib", "numpy"],
  python_requires=">=3.6",
)



from PyQt5.QtWidgets import (QWidget, QLabel, QLineEdit, QPushButton, QVBoxLayout,
                             QGridLayout, QHBoxLayout, QSpacerItem, QSizePolicy)
from PyQt5.QtGui import QFont, QCursor
from PyQt5.QtCore import Qt, pyqtSignal
from .spoonacular import FoodDatabase

class FoodDatabasePanel(QWidget):
  search_signal = pyqtSignal(str)
  emit_search_results = pyqtSignal(object)

  def __init__(self, parent):
    super().__init__(parent)
    self.create_panel()
    self.setStyleSheet("QLabel{color:white;}")

  def create_panel(self):
    grid = QGridLayout()
    
    description_layout = QVBoxLayout()
    find_food_label = QLabel("Find Food")
    find_food_label.setFont(QFont("Ariel", 50))
    find_food_label.setAlignment(Qt.AlignCenter)
    
    description_label = QLabel(
    """<html><head/><body><p align="center">Lorem ipsum dolor sit amet, consectetur adipiscing elit.
       </p><p align="center">Nulla congue consequat ante, vitae viverra quam pharetra non.
       </p><p align="center">Hac habitasse platea dictumst. Aliquam eu mi imperdiet.</p></body></html>
    """)
    
    description_layout.addWidget(find_food_label)
    description_layout.addWidget(description_label)

    search_layout = QHBoxLayout()
    self.search_bar = QLineEdit()
    self.search_bar.setStyleSheet("background-color:white;")
    self.search_bar.setPlaceholderText("Search")
    spacer = QSpacerItem(40, 20, QSizePolicy.Expanding)
    spacer1 = QSpacerItem(40, 20, QSizePolicy.Expanding)

    search_layout.addSpacerItem(spacer)
    search_layout.addWidget(self.search_bar)
    search_layout.addSpacerItem(spacer1)
    
    search_button_layout = QHBoxLayout()
    search_button = QPushButton("Search")
    search_button.setStyleSheet("color:white;background-color: #440D0F;")
    search_button.setCursor(QCursor(Qt.PointingHandCursor))
    search_button.clicked.connect(self.emit_search_signal)

    spacer2 = QSpacerItem(40, 20, QSizePolicy.Expanding)
    spacer3 = QSpacerItem(40, 20, QSizePolicy.Expanding)
    search_button_layout.addSpacerItem(spacer2)
    search_button_layout.addWidget(search_button)
    search_button_layout.addSpacerItem(spacer3)

    grid.addLayout(description_layout, 0, 0)
    grid.addLayout(search_layout, 1, 0)
    grid.addLayout(search_button_layout, 2, 0)
    self.setLayout(grid)

  def emit_search_signal(self):
    if not self.search_bar.text() == '':
      api = FoodDatabase()
      search = self.search_bar.text()
      search_results = api.food_search(search, 6)
      api.download_food_images(search_results, 250)
      self.emit_search_results.emit([search_results, search])
      self.search_signal.emit("Search")



from PyQt5.QtWidgets import (QWidget, QFrame, QFormLayout, QGridLayout,
                            QVBoxLayout, QLabel, QGroupBox, QRadioButton,
                            QHBoxLayout, QLineEdit, QPushButton, QTableWidget,
                            QTableWidgetItem, QHeaderView)
from PyQt5.QtGui import QFont, QCursor
from PyQt5.QtCore import Qt
from .body_fat_calculator import BodyFatCalculator
from fitness_tracker.user_profile.profile_db import fetch_units

class MainPanel(QWidget):
  def __init__(self, parent):
    super().__init__(parent)
    self.create_panel()

  def create_panel(self):
    grid = QGridLayout()
    grid.addLayout(self.create_description(), 0, 0, 1, 2)
    grid.addLayout(self.create_calculator(), 1, 0, 1, 1)
    grid.addLayout(self.create_table(), 1, 1, 1, 1)
    self.setLayout(grid)

  def create_description(self):
    description_layout = QVBoxLayout()
    
    description_label = QLabel("Body Fat Calculator estimates your body fat percentage based on your measurements.", self)
    description_label.setFixedHeight(70)
    description_layout.addWidget(description_label)
    return description_layout

  def create_calculator(self):
    title_frame = QFrame()
    title_layout = QVBoxLayout()

    calculator_label = QLabel("Calculate your body fat", self)
    calculator_label.setFont(QFont("Ariel", 15))
    calculator_label.setFixedHeight(70)

    title_layout.addWidget(calculator_label)
    title_frame.setLayout(title_layout)
    
    calculator_frame = QFrame()
    calculator_frame.setFrameStyle(QFrame.StyledPanel)
    calculator_frame.setFixedWidth(300)
     
    form_layout = self.create_form_metric() if fetch_units() == "metric" else self.create_form_imperial()
    calculator_frame.setLayout(form_layout)

    wrapper_layout = QVBoxLayout()
    wrapper_layout.addWidget(title_frame)
    wrapper_layout.addWidget(calculator_frame)
    return wrapper_layout

  def create_form_metric(self):
    self.calculator_layout = QFormLayout()
    
    gender_label = QLabel("Gender", self)
    
    gbox = QGroupBox()
    vbox = QHBoxLayout()
    vbox.setAlignment(Qt.AlignCenter)
    
    self.male_button = QRadioButton("male")
    self.male_button.setChecked(True)
    self.female_button = QRadioButton("female")
    
    self.male_button.toggled.connect(self.hide_or_show_hip)

    vbox.addWidget(self.male_button)
    vbox.addWidget(self.female_button)
    gbox.setLayout(vbox)

    age_label = QLabel("Age", self)
    self.age_entry = QLineEdit()
    
    weight_label = QLabel("Weight", self)
    self.weight_entry = QLineEdit()
    self.weight_entry.setPlaceholderText("kg")

    height_label = QLabel("Height", self)
    self.height_entry = QLineEdit()
    self.height_entry.setPlaceholderText("cm")

    neck_label = QLabel("Neck", self)
    self.neck_entry = QLineEdit()
    self.neck_entry.setPlaceholderText("cm")
    
    waist_label = QLabel("Waist", self)
    self.waist_entry = QLineEdit()
    self.waist_entry.setPlaceholderText("cm")

    self.calculate_button = QPushButton("Calculate", self)
    self.calculate_button.setCursor(QCursor(Qt.PointingHandCursor))
    self.calculate_button.clicked.connect(self.calculate)

    self.calculator_layout.addRow(gender_label, gbox)
    self.calculator_layout.addRow(age_label, self.age_entry)
    self.calculator_layout.addRow(weight_label, self.weight_entry)
    self.calculator_layout.addRow(height_label, self.height_entry)
    self.calculator_layout.addRow(neck_label, self.neck_entry)
    self.calculator_layout.addRow(waist_label, self.waist_entry)
    self.calculator_layout.addRow(self.calculate_button)
    return self.calculator_layout
  
  def create_form_imperial(self):
    self.calculator_layout = QFormLayout()
    
    gender_label = QLabel("Gender", self)
    
    gbox = QGroupBox()
    vbox = QHBoxLayout()
    vbox.setAlignment(Qt.AlignCenter)
    
    self.male_button = QRadioButton("male")
    self.male_button.setChecked(True)
    self.female_button = QRadioButton("female")
    
    self.male_button.toggled.connect(self.hide_or_show_hip)

    vbox.addWidget(self.male_button)
    vbox.addWidget(self.female_button)
    gbox.setLayout(vbox)

    age_label = QLabel("Age", self)
    self.age_entry = QLineEdit()
    
    weight_label = QLabel("Weight", self)
    self.weight_entry = QLineEdit()
    self.weight_entry.setPlaceholderText("pounds")

    height_layout = QHBoxLayout()
    height_label = QLabel("Height", self)
    self.height_feet_entry = QLineEdit()
    self.height_feet_entry.setPlaceholderText("feet")
    self.height_inches_entry = QLineEdit()
    self.height_inches_entry.setPlaceholderText("inches")
    
    height_layout.addWidget(self.height_feet_entry)
    height_layout.addWidget(self.height_inches_entry)
    
    neck_layout = QHBoxLayout()
    neck_label = QLabel("Neck", self)
    self.neck_feet_entry = QLineEdit()
    self.neck_feet_entry.setPlaceholderText("feet")
    self.neck_inches_entry = QLineEdit()
    self.neck_inches_entry.setPlaceholderText("inches")
    
    neck_layout.addWidget(self.neck_feet_entry)
    neck_layout.addWidget(self.neck_inches_entry)
    
    waist_layout = QHBoxLayout()
    waist_label = QLabel("Waist", self)
    self.waist_feet_entry = QLineEdit()
    self.waist_feet_entry.setPlaceholderText("feet")
    self.waist_inches_entry = QLineEdit()
    self.waist_inches_entry.setPlaceholderText("inches")
    
    waist_layout.addWidget(self.waist_feet_entry)
    waist_layout.addWidget(self.waist_inches_entry)

    self.calculate_button = QPushButton("Calculate", self)
    self.calculate_button.setCursor(QCursor(Qt.PointingHandCursor))
    self.calculate_button.clicked.connect(self.calculate)

    self.calculator_layout.addRow(gender_label, gbox)
    self.calculator_layout.addRow(age_label, self.age_entry)
    self.calculator_layout.addRow(weight_label, self.weight_entry)
    self.calculator_layout.addRow(height_label, height_layout)
    self.calculator_layout.addRow(neck_label, neck_layout)
    self.calculator_layout.addRow(waist_label, waist_layout)
    self.calculator_layout.addRow(self.calculate_button)

    return self.calculator_layout

  def hide_or_show_hip(self):
    units = fetch_units()
    
    if self.female_button.isChecked():
      self.calculator_layout.removeWidget(self.calculate_button)
      self.calculate_button.deleteLater()
      
      self.calculate_button = QPushButton("Calculate", self)
      self.calculate_button.setCursor(QCursor(Qt.PointingHandCursor))
      self.calculate_button.clicked.connect(self.calculate)
      
      self.hip_label = QLabel("Hip", self)
      
      if units == "metric":
        self.hip_entry = QLineEdit()
        self.hip_entry.setPlaceholderText("cm")
        self.calculator_layout.addRow(self.hip_label, self.hip_entry)
      elif units == "imperial":
        self.hip_layout = QHBoxLayout()
        self.hip_feet_entry = QLineEdit()
        self.hip_feet_entry.setPlaceholderText("feet")
        self.hip_inches_entry = QLineEdit()
        self.hip_inches_entry.setPlaceholderText("inches")
        
        self.hip_layout.addWidget(self.hip_feet_entry)
        self.hip_layout.addWidget(self.hip_inches_entry)
        self.calculator_layout.addRow(self.hip_label, self.hip_layout)

      self.calculator_layout.addRow(self.calculate_button)
    else:
      if units == "metric":
        self.calculator_layout.removeWidget(self.hip_label)
        self.hip_label.deleteLater()
        self.calculator_layout.removeWidget(self.hip_entry)
        self.hip_entry.deleteLater()
      elif units == "imperial":
        self.calculator_layout.removeWidget(self.hip_label)
        self.hip_label.deleteLater()
        self.calculator_layout.removeWidget(self.hip_feet_entry)
        self.hip_feet_entry.deleteLater()
        self.calculator_layout.removeWidget(self.hip_inches_entry)
        self.hip_inches_entry.deleteLater()
  
  def create_table(self):
    frame = QFrame()
    table_layout = QVBoxLayout()

    frame.setLayout(table_layout)

    self.table = QTableWidget(7, 2)
    self.table.setEditTriggers(QTableWidget.NoEditTriggers) 
    self.table.verticalHeader().setVisible(False)
    self.table.horizontalHeader().setVisible(False)
    
    rows = [["Body Fat (U.S. Navy Method)", ""],
            ["Body Fat Category", ""],
            ["Body Fat Mass", ""],
            ["Lean Body Mass", ""],
            ["Ideal Body Fat for Given Age (Jackson & Pollard)", ""],
            ["Body Fat to Lose to Reach Ideal", ""],
            ["Body Fat (BMI method)", ""]]
    
    i = j = 0
    for row in rows:
      item1 = QTableWidgetItem(row[0])
      item2 = QTableWidgetItem(row[1])
      item1.setTextAlignment(Qt.AlignCenter)
      item2.setTextAlignment(Qt.AlignCenter)
      self.table.setItem(i, 0, item1)
      self.table.setItem(j, 1, item2)
      i += 1
      j += 1
    
    for i in range(2):
      self.table.horizontalHeader().setSectionResizeMode(i, QHeaderView.Stretch)
    
    for i in range(7):
      self.table.verticalHeader().setSectionResizeMode(i, QHeaderView.Stretch)

    grid = QGridLayout()
    grid.addWidget(frame, 0, 0)
    grid.addWidget(self.table, 1, 0)
    return grid

  def calculate(self):
    gender = "male" if self.male_button.isChecked() else "female"
    units = "kg" if fetch_units() == "metric" else "lbs"
    m = self.get_measurements()
    try:
      calc = BodyFatCalculator(m["Gender"], m["Age"], m["Weight"], m["Height"], m["Neck"], m["Waist"], m["Units"], m["Hip"])
      results = calc.get_results()
      self.set_results(results, units)
    except TypeError: # dict values are None
      pass
  
  def get_measurements(self):
    measurements = {}
    measurements["Gender"] = "male" if self.male_button.isChecked() else "female"
    units = fetch_units()
    try:
      measurements["Age"] = int(self.age_entry.text())
      measurements["Weight"] = int(self.weight_entry.text())
      measurements["Units"] = units
      if units == "metric":
        measurements["Height"] = int(self.height_entry.text())
        measurements["Neck"] = int(self.neck_entry.text())
        measurements["Waist"] = int(self.waist_entry.text())
        try:
          measurements["Hip"] = int(self.hip_entry.text())
        except (AttributeError, RuntimeError): # hip entry might not exist
          measurements["Hip"] = None
      elif units == "imperial":
        measurements["Height"] = int(self.height_feet_entry.text())*12+float(self.height_inches_entry.text())
        measurements["Neck"] = int(self.neck_feet_entry.text())*12+float(self.neck_inches_entry.text())
        measurements["Waist"] = int(self.waist_feet_entry.text())*12+float(self.waist_inches_entry.text())
        try: measurements["Hip"] = int(self.hip_feet_entry.text())*12+float(self.hip_inches_entry.text())
        except (AttributeError, RuntimeError): measurements["Hip"] = None
      return measurements
    except ValueError: # values inside entries aren't numbers
      pass
  
  def set_results(self, results, units):
    if results["Body Fat Navy"] <= 0: return
    self.table.item(0, 1).setText("".join([str(results["Body Fat Navy"]), "%"]))
    self.table.item(1, 1).setText(str(results["Body Fat Category"]))
    self.table.item(2, 1).setText(" ".join([str(results["Fat Mass"]), units]))
    self.table.item(3, 1).setText(" ".join([str(results["Lean Body Mass"]), units]))
    self.table.item(4, 1).setText(str(results["Ideal Body Fat"]))
    self.table.item(5, 1).setText(" ".join([str(results["Body Fat To Lose To Reach Ideal"]), units]))
    self.table.item(6, 1).setText("".join([str(results["Body Fat BMI"]), "%"]))



import json
import os
from PyQt5.QtWidgets import QLabel, QWidget, QPushButton, QFormLayout, QLineEdit, QHBoxLayout
from PyQt5.QtCore import pyqtSignal
from .big_lifts_db import (fetch_preferred_lifts, fetch_one_rep_maxes, lift_difference,
                           update_lift_history, update_1RM_lifts)
from fitness_tracker.user_profile.profile_db import fetch_units

class Update1RMWindow(QWidget):
  change_1RM_lifts_signal = pyqtSignal(bool)
  history_signal = pyqtSignal(bool)

  def __init__(self):
    super().__init__()
    self.units = "kg" if fetch_units() == "metric" else "lb"
    self.preferred_lifts = json.loads(fetch_preferred_lifts())
    self.setWindowTitle("Update One Rep Max Lifts")
    self.setLayout(self.create_panel())
    self.set_line_edit_values()

  def create_panel(self):
    form_layout = QFormLayout()

    exercise_label = QLabel("Exercise")
    weight_label = QLabel("Weight")
    
    horizontal_press_label = QLabel(self.preferred_lifts["Horizontal Press"])
    self.horizontal_press_edit = QLineEdit()
    units_label = QLabel(self.units)
    hbox = QHBoxLayout()
    hbox.addWidget(self.horizontal_press_edit)
    hbox.addWidget(units_label)

    floor_pull_label = QLabel(self.preferred_lifts["Floor Pull"])
    self.floor_pull_edit = QLineEdit()
    units_label1 = QLabel(self.units)
    hbox1 = QHBoxLayout()
    hbox1.addWidget(self.floor_pull_edit)
    hbox1.addWidget(units_label1)

    squat_label = QLabel(self.preferred_lifts["Squat"])
    self.squat_edit = QLineEdit()
    units_label2 = QLabel(self.units)
    hbox2 = QHBoxLayout()
    hbox2.addWidget(self.squat_edit)
    hbox2.addWidget(units_label2)

    vertical_press_label = QLabel(self.preferred_lifts["Vertical Press"])
    self.vertical_press_edit = QLineEdit()
    units_label3 = QLabel(self.units)
    hbox3 = QHBoxLayout()
    hbox3.addWidget(self.vertical_press_edit)
    hbox3.addWidget(units_label3)

    save_button = QPushButton("Save")
    save_button.clicked.connect(lambda: self.save_1RM_lifts())
    cancel_button = QPushButton("Cancel")
    cancel_button.clicked.connect(lambda: self.close_update_1RM())

    form_layout.addRow(exercise_label, weight_label)
    form_layout.addRow(horizontal_press_label, hbox)
    form_layout.addRow(floor_pull_label, hbox1)
    form_layout.addRow(squat_label, hbox2)
    form_layout.addRow(vertical_press_label, hbox3)
    form_layout.addRow(save_button, cancel_button)
    return form_layout

  def save_1RM_lifts(self):
    try:
      exercises = list(json.loads(fetch_one_rep_maxes()).keys())
      horizontal_press_max = str(float(self.horizontal_press_edit.text()))
      floor_pull_max = str(float(self.floor_pull_edit.text()))
      squat_max = str(float(self.squat_edit.text()))
      vertical_press_max = str(float(self.vertical_press_edit.text()))
      new_maxes = {exercises[0]:horizontal_press_max, exercises[1]:floor_pull_max,
                   exercises[2]:squat_max, exercises[3]:vertical_press_max}
      
      diff = lift_difference(new_maxes, one_RM=True)
      update_lift_history(diff) 
      self.history_signal.emit(True)
      update_1RM_lifts(new_maxes)
      self.change_1RM_lifts_signal.emit(True)
      self.set_line_edit_values()
      self.close() 
    except ValueError: # user submitted text/empty string
      pass
  
  def close_update_1RM(self):
    self.close()
    self.set_line_edit_values()

  def set_line_edit_values(self):
    one_rep_maxes = list(json.loads(fetch_one_rep_maxes()).values())
    self.horizontal_press_edit.setText(one_rep_maxes[0])
    self.floor_pull_edit.setText(one_rep_maxes[1])
    self.squat_edit.setText(one_rep_maxes[2])
    self.vertical_press_edit.setText(one_rep_maxes[3])



from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QLabel, QFrame, QLineEdit,
                             QPushButton, QTableWidget, QTableWidgetItem, QHeaderView)
from PyQt5.QtGui import QFont, QCursor
from PyQt5.QtCore import Qt
from .one_rep_max_calculator import OneRepMaxCalculator
from fitness_tracker.user_profile.profile_db import fetch_units
from fitness_tracker.config import db_path

class MainPanel(QWidget):
  def __init__(self, parent):
    super().__init__()
    self.create_panel()

  def create_panel(self):
    grid = QGridLayout()
    grid.addLayout(self.create_description(), 0, 0, 1, 2)
    grid.addLayout(self.create_calculator(), 1, 0, 1, 1)
    grid.addLayout(self.create_progression(), 1, 1, 2, 1)
    self.setLayout(grid)

  def create_description(self):
    description_layout = QVBoxLayout()

    description_label = QLabel(""" One Rep Max calculator can calculate your estimated maximum repetitions based on your recent heavy lift. 
 Calculator will also show suggested progression towards that one rep max. """, self)
    description_label.setStyleSheet("border: 1px solid black;")
    description_label.setFixedHeight(70)
    description_layout.addWidget(description_label)

    return description_layout

  def create_calculator(self):
    title_frame = QFrame()
    title_layout = QVBoxLayout()
    
    calculator_label = QLabel("Calculate your one rep max", self)
    calculator_label.setFont(QFont("Ariel", 15))

    title_layout.addWidget(calculator_label)
    title_frame.setLayout(title_layout)
    
    calculator_layout = QVBoxLayout()
    
    calculator_frame = QFrame()
    calculator_frame.setFrameStyle(QFrame.StyledPanel)
    calculator_frame.setFixedWidth(300) 
    
    weight_label = QLabel("Weight", self)
    self.weight_entry = QLineEdit()
    
    reps_label = QLabel("Repetitions", self)
    self.reps_entry = QLineEdit()
    
    calculate_button = QPushButton("Calculate", self)
    calculate_button.setCursor(QCursor(Qt.PointingHandCursor))
    calculate_button.clicked.connect(self.calculate)

    calculator_layout.addWidget(weight_label)
    calculator_layout.addWidget(self.weight_entry)
    calculator_layout.addWidget(reps_label)
    calculator_layout.addWidget(self.reps_entry)
    calculator_layout.addWidget(calculate_button)
    
    calculator_frame.setLayout(calculator_layout)
    
    self.result_label = QLabel(self)
    
    wrapper_layout = QVBoxLayout()
    wrapper_layout.addWidget(title_frame)
    wrapper_layout.addWidget(calculator_frame)
    wrapper_layout.addWidget(self.result_label)
    
    return wrapper_layout

  def create_progression(self):
    frame = QFrame()
    progression_layout = QVBoxLayout()
    
    progression_label = QLabel("Progression to your max", self)
    progression_label.setAlignment(Qt.AlignCenter)
    progression_label.setFont(QFont("Ariel", 15))

    progression_layout.addWidget(progression_label)
    frame.setLayout(progression_layout)
    
    self.table = QTableWidget(7,5)
    self.table.setEditTriggers(QTableWidget.NoEditTriggers)
    self.table.verticalHeader().setVisible(False)
    
    table_items = {"table_headers": ["Set", "%", "Weight", "Reps", "Rest"], 
                   "sets": ["1", "2", "3", "4", "5", "6", "7", "8"],
                   "percentages": ["~30%-50%", "~60%", "~70", "~80%", "~90%", "~100%", "~102%"],
                   "weight": [""]*7,
                   "number_of_reps": ["8", "5", "3", "1", "1", "1", "1"],
                   "rest": ["~2 min", "~2 min", "~3 min", "~3 min", "~5 min", "~5-15 min", "~5-15 min"]}
    
    for key, value in table_items.items():
      for i, item in enumerate(value):
        if key == "table_headers":
          self.table.setHorizontalHeaderItem(i, QTableWidgetItem(item))
          self.table.horizontalHeader().setSectionResizeMode(i, QHeaderView.Stretch)
        else:
          _item = QTableWidgetItem(item)
          _item.setTextAlignment(Qt.AlignCenter)
          if key == "sets": self.table.setItem(i, 0, _item)
          elif key == "percentages": self.table.setItem(i, 1, _item)
          elif key == "weight": self.table.setItem(i, 2, _item)
          elif key == "number_of_reps": self.table.setItem(i, 3, _item)
          elif key == "rest": self.table.setItem(i, 4, _item)

    for i in range(7):
      self.table.verticalHeader().setSectionResizeMode(i, QHeaderView.Stretch)
    
    grid = QGridLayout()
    grid.addWidget(frame, 0, 0)
    grid.addWidget(self.table, 1, 0, 1, 1)
    return grid

  def calculate(self):
    try:
      units = fetch_units()
      weight = self.weight_entry.text()
      repetitions = self.reps_entry.text()
      units = "kg" if units == "metric" else "lb"
      
      calc = OneRepMaxCalculator(weight, repetitions)
      results = calc.results()
      self.result_label.setText("".join(["Your estimated one rep max is: ", str(results[5]), " ", units]))
      j = 0
      for i in range(7):
        self.table.item(i, 2).setText("".join([str(results[j]), " ", units]))
        j += 1
    except ValueError:
      pass



from functools import partial
from PyQt5.QtWidgets import QWidget, QLabel, QLineEdit, QRadioButton, QFormLayout, QHBoxLayout, QPushButton, QVBoxLayout, QGridLayout
from PyQt5.QtCore import pyqtSignal, Qt, pyqtSlot
from .workouts_db import update_workouts, update_current_workout

class CreateWorkoutWindow1(QWidget):
  continue_signal = pyqtSignal(object)

  def __init__(self):
    super().__init__()
    self.setWindowTitle("Create New Workout")
    self.create_layout()

  def create_layout(self):
    form = QFormLayout()
    workout_name_label = QLabel("Workout name", self)
    self.workout_name_line_edit = QLineEdit()
    
    days_in_a_week_label = QLabel("Days in a week", self)
    self.days_in_a_week = QHBoxLayout()
    days = ["Monday", "Tuesday", "Wednesday", "Thursday",
            "Friday", "Saturday", "Sunday"]
     
    for day in days:
      button = QRadioButton(day)
      button.setAutoExclusive(False)
      self.days_in_a_week.addWidget(button)
    
    set_current_workout_label = QLabel("Set as current workout plan?")
    set_current_workout_layout = QHBoxLayout()
    self.yes = QRadioButton("Yes")
    self.yes.setChecked(True)
    self.no = QRadioButton("No")
    set_current_workout_layout.addWidget(self.yes)
    set_current_workout_layout.addWidget(self.no)
    
    continue_button = QPushButton("Continue")
    continue_button.clicked.connect(lambda: self.emit_continue_signal())
    cancel_button = QPushButton("Cancel")
    cancel_button.clicked.connect(lambda: self.close())

    form.addRow(workout_name_label, self.workout_name_line_edit)
    form.addRow(days_in_a_week_label, self.days_in_a_week)
    form.addRow(set_current_workout_label, set_current_workout_layout)
    form.addRow(cancel_button, continue_button)
    self.setLayout(form)

  def emit_continue_signal(self):
    workout_name = self.workout_name_line_edit.text()
    days = []
    set_current_workout = True if self.yes.isChecked() else False
    for i in range(self.days_in_a_week.count()):
      button = self.days_in_a_week.itemAt(i).widget()
      if button.isChecked():
        days.append(button.text())
    if workout_name != "" and len(days) > 0:
      self.continue_signal.emit([workout_name, days, set_current_workout])

class CreateWorkoutWindow2(QWidget):
  show_edit_workout = pyqtSignal(str)
  refresh_layout_signal = pyqtSignal(bool)
  show_existing_workout_edit = pyqtSignal(object)

  def __init__(self, workout_name, workout_days, set_as_current_workout=False, empty_workout=True):
    super().__init__()
    self.workout_name = workout_name
    self.empty_workout = empty_workout
    self.setWindowTitle(self.workout_name)
    self.set_as_current_workout = set_as_current_workout
    self.workout_days = workout_days
    self.workouts = {}
    self.create_layout()

  def create_layout(self):
    days_layout = QVBoxLayout()
    save_button = QPushButton("Save")
    save_button.clicked.connect(lambda: self.save_workout_plan())
    for day in self.workout_days:
      day_button = QPushButton(day, self)
      if self.empty_workout:
        day_button.clicked.connect(partial(self.show_edit_workout.emit, day))
      else:
        day_button.clicked.connect(partial(self.show_existing_workout_edit.emit, [self.workout_name, day]))
      day_button.setFixedSize(250, 100)
      days_layout.addWidget(day_button)
    days_layout.setAlignment(Qt.AlignCenter)
    days_layout.addWidget(save_button)
    self.setLayout(days_layout)
  
  @pyqtSlot(object)
  def add_workouts(self, workout_day):
    self.workouts[workout_day[0]] = workout_day[1]

  def save_workout_plan(self):
    update_workouts(self.workout_name, self.workouts)
    if self.set_as_current_workout:
      update_current_workout(self.workout_name, self.set_as_current_workout)
    self.refresh_layout_signal.emit(True)
    self.close()

class EditWorkout(QWidget):
  update_workout_day_signal = pyqtSignal(object)

  def __init__(self, day, existing_workout_day_info={}):
    super().__init__()
    self.workout_day = day
    self.workout_info = existing_workout_day_info
    self.setWindowTitle("".join(["Edit Workout ", "- ", self.workout_day]))
    
    i = 0
    while len(self.workout_info) < 6:
      self.workout_info[str(i)] = "empty_exercise"
      i += 1
    
    self.create_layout()

  def create_layout(self):
    grid_layout = QGridLayout() 
   
    exercise_label = QLabel("Exercise")
    sets_label = QLabel("Sets")
    reps_label = QLabel("Reps")
    rest_label = QLabel("Rest")
    duration_label = QLabel("Duration (optional)")
    
    self.labels = [exercise_label, sets_label, reps_label,
                   rest_label, duration_label]

    labels_layout = QHBoxLayout()
    for label in self.labels:
      labels_layout.addWidget(label)
    
    grid_layout.addLayout(labels_layout, 0, 5)
    
    self.row_layouts = [None] * 6
    number_labels = [None] * 6
    exercise_line_edits = [None] * 6
    sets_line_edits = [None] * 6
    reps_line_edits = [None] * 6
    rest_line_edits = [None] * 6
    duration_line_edits = [None] * 6
    
    row = 1
    i = 0
    for exercise in self.workout_info:
      self.row_layouts[i] = QHBoxLayout()
      number_labels[i] = QLabel(str(i+1)+".")
      exercise_line_edits[i] = QLineEdit()
      sets_line_edits[i] = QLineEdit()
      reps_line_edits[i] = QLineEdit()
      rest_line_edits[i] = QLineEdit()
      duration_line_edits[i] = QLineEdit()
      if self.workout_info[exercise] != "empty_exercise":
        exercise_line_edits[i].setText(exercise)
        sets_line_edits[i].setText(self.workout_info[exercise]["Sets"])
        reps_line_edits[i].setText(self.workout_info[exercise]["Reps"])
        rest_line_edits[i].setText(self.workout_info[exercise]["Rest"])
        duration_line_edits[i].setText(self.workout_info[exercise]["Duration (optional)"])

      self.row_layouts[i].addWidget(number_labels[i])
      self.row_layouts[i].addWidget(exercise_line_edits[i])
      self.row_layouts[i].addWidget(sets_line_edits[i])
      self.row_layouts[i].addWidget(reps_line_edits[i])
      self.row_layouts[i].addWidget(rest_line_edits[i])
      self.row_layouts[i].addWidget(duration_line_edits[i])
      
      grid_layout.addLayout(self.row_layouts[i], row, 5)
      row += 1
      i += 1

    save_button = QPushButton("Save")
    save_button.clicked.connect(lambda: self.save_workout())
    cancel_button = QPushButton("Cancel")
    cancel_button.clicked.connect(lambda: self.close())
    
    grid_layout.addWidget(cancel_button, row, 2)
    grid_layout.addWidget(save_button, row, 3)
    self.setLayout(grid_layout)

  def save_workout(self):
    workout = {}
    for row in self.row_layouts:
      exercise_name = ""
      for i in range(row.count()):
        if i != 0: # ignore number_label
          label = self.labels[i-1].text()
          row_widget = row.itemAt(i).widget()
          if exercise_name == "" and label == "Exercise":
            exercise_name = row_widget.text()
            if exercise_name == "": break
            else: workout[exercise_name] = {}
          else:
            workout[exercise_name][label] = row_widget.text()
    self.update_workout_day_signal.emit([self.workout_day, workout])
    self.close()
